# Global Change Assessment Model (GCAM)

The Joint Global Change Research Institute (JGCRI) is the home and
primary development institution for GCAM, an integrated assessment
tool for exploring consequences and responses to global
change. Climate change is a global issue that impacts all regions of
the world and all sectors of the global economy. Thus, any responses
to the threat of climate change, such as policies or international
agreements to limit greenhouse gas emissions, can have wide ranging
consequences throughout the energy system as well as on land use and
land cover. Integrated assessment models endeavor to represent all
world regions and all sectors of the economy in an economic framework
in order to explore interactions between sectors and understand the
potential ramifications of climate mitigation actions.

GCAM has been developed at PNNL for over 20 years and is now a freely
available community model and documented online (See below). The team
at JGCRI is comprised of economists, engineers, energy experts, forest
ecologists, agricultural scientists, and climate system scientists who
develop the model and apply it to a range of science and policy
questions and work closely with Earth system and ecosystem modelers to
integrate the human decision components of GCAM into their analyses.

## Model Overview

GCAM is a dynamic-recursive model with technology-rich representations
of the economy, energy sector, land use and water linked to a climate
model that can be used to explore climate change mitigation policies
including carbon taxes, carbon trading, regulations and accelerated
deployment of energy technology. Regional population and labor
productivity growth assumptions drive the energy and land-use systems
employing numerous technology options to produce, transform, and
provide energy services as well as to produce agriculture and forest
products, and to determine land use and land cover. Using a run period
extending from 1990 – 2100 at 5 year intervals, GCAM has been used to
explore the potential role of emerging energy supply technologies and
the greenhouse gas consequences of specific policy measures or energy
technology adoption including; CO2 capture and storage, bioenergy,
hydrogen systems, nuclear energy, renewable energy technology, and
energy use technology in buildings, industry and the transportation
sectors. GCAM is an Representative Concentration Pathway (RCP)-class
model. This means it can be used to simulate scenarios, policies, and
emission targets from various sources including the Intergovernmental
Panel on Climate Change (IPCC). Output includes projections of future
energy supply and demand and the resulting greenhouse gas emissions,
radiative forcing and climate effects of 16 greenhouse gases, aerosols
and short-lived species at 0.5×0.5 degree resolution, contingent on
assumptions about future population, economy, technology, and climate
mitigation policy.

## Documentation

* [GCAM Documentation](http://jgcri.github.io/gcam-doc/)
* [Getting Started with GCAM](http://jgcri.github.io/gcam-doc/user-guide.html)
* [GCAM Community](http://www.globalchange.umd.edu/models/gcam/gcam-community/)
* [2015 Tutorial Slides](http://www.globalchange.umd.edu/data/annual-meetings/2015/GCAM_Tutorial_2015.pdf)
* GCAM lectures from the UMD Water and Climate class
  * [Lecture 3: Introduction to GCAM](https://www.youtube.com/watch?v=xRF9lFwtMr0)
  * [Lecture 4: GCAM Tutorial](https://www.youtube.com/watch?v=S7vAShH-dbs)

## Selected Publications

Edmonds, J., and J. Reilly (1985)Global Energy: Assessing the Future (Oxford University Press, New York) pp.317.

Edmonds, J., M. Wise, H. Pitcher, R. Richels, T. Wigley, and C. MacCracken. (1997) “An Integrated Assessment of Climate Change and the Accelerated Introduction of Advanced Energy Technologies”, Mitigation and Adaptation Strategies for Global Change, 1, pp. 311-39

Kim, S.H., J. Edmonds, J. Lurz, S. J. Smith, and M. Wise (2006) “The ObjECTS Framework for Integrated Assessment: Hybrid Modeling of Transportation ” Energy Journal (Special Issue #2) pp 51-80.

[Full list of GCAM publications](http://jgcri.github.io/gcam-doc/references.html)
