#ifndef _MERGE_RUNNER_H_
#define _MERGE_RUNNER_H_
#if defined(_MSC_VER)
#pragma once
#endif

/*
* LEGAL NOTICE
* This computer software was prepared by Battelle Memorial Institute,
* hereinafter the Contractor, under Contract No. DE-AC05-76RL0 1830
* with the Department of Energy (DOE). NEITHER THE GOVERNMENT NOR THE
* CONTRACTOR MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR ASSUMES ANY
* LIABILITY FOR THE USE OF THIS SOFTWARE. This notice including this
* sentence must appear on any copies of this computer software.
* 
* EXPORT CONTROL
* User agrees that the Software will not be shipped, transferred or
* exported into any country or used in any manner prohibited by the
* United States Export Administration Act or any other applicable
* export laws, restrictions or regulations (collectively the "Export Laws").
* Export of the Software may require some form of license or other
* authority from the U.S. Government, and failure to obtain such
* export control license may result in criminal liability under
* U.S. laws. In addition, if the Software is identified as export controlled
* items under the Export Laws, User represents and warrants that User
* is not a citizen, or otherwise located within, an embargoed nation
* (including without limitation Iran, Syria, Sudan, Cuba, and North Korea)
*     and that User is not otherwise prohibited
* under the Export Laws from receiving the Software.
* 
* Copyright 2011 Battelle Memorial Institute.  All Rights Reserved.
* Distributed as open-source under the terms of the Educational Community 
* License version 2.0 (ECL 2.0). http://www.opensource.org/licenses/ecl2.php
* 
* For further details, see: http://www.globalchange.umd.edu/models/gcam/
*
*/



/*! 
* \file merge_runner.h
* \ingroup Objects
* \brief The ScenarioRunner class header file.
* \author Josh Lurz
*/

#include <memory>
#include <string>
#include <list>
#include "containers/include/iscenario_runner.h"

class Scenario;
class Timer;

/*! 
* \ingroup Objects
* \brief A class which runs the scenario only to get the merged output.
* \author Josh Lurz
*/
class MergeRunner: public IScenarioRunner {
	friend class ScenarioRunnerFactory;
public:
    virtual ~MergeRunner();

    virtual const std::string& getName() const;

    // IParsable interface
    virtual bool XMLParse( const xercesc::DOMNode* aRoot );

    virtual bool setupScenarios( Timer& timer,
                                 const std::string aName,
                                 const std::list<std::string> aScenComponents =
                                   std::list<std::string>() );
    
    virtual bool runScenarios( const int aSinglePeriod,
                               const bool aPrintDebugging,
                               Timer& aTimer );
    
    void printOutput( Timer& timer, const bool aCloseDB = true ) const;

    virtual void cleanup();
	Scenario* getInternalScenario();
	virtual const Scenario* getInternalScenario() const;

protected:
    MergeRunner();
	static const std::string& getXMLNameStatic();
    
	//! The internal scenario.
	std::auto_ptr<Scenario> mScenario;
};
#endif // _MERGE_RUNNER_H_
