# LEGAL NOTICE
# This computer software was prepared by Battelle Memorial Institute,
# hereinafter the Contractor, under Contract No. DE-AC05-76RL0 1830
# with the Department of Energy (DOE). NEITHER THE GOVERNMENT NOR THE
# CONTRACTOR MAKES ANY WARRANTY, EXPRESS OR IMPLIED, OR ASSUMES ANY
# LIABILITY FOR THE USE OF THIS SOFTWARE. This notice including this
# sentence must appear on any copies of this computer software.
# 
# EXPORT CONTROL
# User agrees that the Software will not be shipped, transferred or
# exported into any country or used in any manner prohibited by the
# United States Export Administration Act or any other applicable
# export laws, restrictions or regulations (collectively the "Export Laws").
# Export of the Software may require some form of license or other
# authority from the U.S. Government, and failure to obtain such
# export control license may result in criminal liability under
# U.S. laws. In addition, if the Software is identified as export controlled
# items under the Export Laws, User represents and warrants that User
# is not a citizen, or otherwise located within, an embargoed nation
# (including without limitation Iran, Syria, Sudan, Cuba, and North Korea)
#     and that User is not otherwise prohibited
# under the Export Laws from receiving the Software.
# 
# Copyright 2011 Battelle Memorial Institute.  All Rights Reserved.
# Distributed as open-source under the terms of the Educational Community 
# License version 2.0 (ECL 2.0). http://www.opensource.org/licenses/ecl2.php
# 
# For further details, see: http://www.globalchange.umd.edu/models/gcam/
#

# color_schemes.R
#
# An automated graphing system to process GCAM output data (as generated by the
# ModelInterface) and generate both standard and user-defined graphs.
#
# Ben Bond-Lamberty, November 2012

# -----------------------------------------------------------------------------
# Various color schemes to try to standardize figures.

# General purpose color scheme where sequentail colors do not clash too much
cbPalette <- c("#999999", "#E69F00", "#56B4E9", "#009E73", "#F0E442", "#0072B2", "#D55E00", "#CC79A7",
"#333333", "#FFCC00", "#CC6600", "#006600", "#3333CC", "#CC0033", "#0099CC", "#999966")

# Color scheme for electricity generation by aggregate fuel
elec_tech_colors <- c( "a Coal" = "#a0237c",
    "b Coal w/CCS" = "#dab4c7",
    "c Gas" = "#25a9e0",
    "d Gas w/CCS" = "#84e7f9",
    "e Oil" = "#d01c2a",
    "f Oil w/CCS" = "#f7988f",
    "g Biomass" = "#00931d",
    "h Biomass w/CCS" = "#88c892",
    "i Nuclear" = "#ef8e27",
    "j Geothermal" = "#ad440c",
    "k Hydro" = "#fdfa28",
    "l Wind" = "#3d86f9",
    "m Solar" = "#fdd67b",
    "n CHP" = "#507fab",
    "o Battery" = "#92a75d",
    "energy reduction" = "grey")

elec_renew_colors <- c( "liquids" = "#d01c2a",
"gas" = "#25a9e0",
"coal" = "#a0237c",
"biomass" = "#00931d",
"nuclear" = "#ef8e27",
"hydro" = "#fdfa28",
"wind w/ backup" = "#3e85fd",
"wind w/ storage" = "#9ec1fd",
"csp w/ backup" = "#ec931a",
"csp w/ storage" = "#f5c88c",
"pv w/ backup" = "#ffd125",
"pv w/ storage" = "#ffe791",
"rooftop solar" = "#ff7a17",
"geothermal" = "#aa440a",
"cogen" = "#507fab",
"battery" = "#92a75d",
"energy reduction" = "grey")

# Buildings colors by fuel
building_colors <- c( "liquids" = "#d01c2a",
    "gas" = "#25a9e0",
    "coal" = "#a0237c",
    "biomass" = "#00931d",
    "electricity" = "#ef8e27",
    "trad biomass" = "#11d081")

# Transportation colors by fuel
trn_fuel_colors <- c( "liquids" = "#d01c2a",
    "gas" = "#25a9e0",
    "coal" = "#a0237c",
    "biomass" = "#00931d",
    "electricity" = "#ef8e27",
    "hydrogen" = "#fffbac")

# Generic end-use color scheme by numbered fuel
enduse_fuel_numbered <- c( "1 liquids" = "#d01c2a",
    "2 gas" = "#25a9e0",
    "3 coal" = "#a0237c",
    "4 biomass" = "#00931d",
    "5 electricity" = "#ef8e27",
    "6 hydrogen" = "#fffbac",
    "7 trad biomass" = "#11d081",
    "8 district heat" = "#b14d38",
    "feedstocks" = "#ff7467",
    "energy reduction" = "grey")

# Color scheme for the aggregate final demands
enduse_colors <- c( "building" = "#facda4", "industry" = "#cef4d1", "transportation" = "#d0f6f7",
    "energy reduction" = "grey")

# Primary energy colors including CCS
PAL_pri_ene <- c(
    "a oil" = "#d01c2a",
    "a oil CCS" = "#f7988f",
    "b natural gas" = "#25a9e0",
    "b natural gas CCS" = "#84e7f9",
    "c coal" = "#a0237c",
    "c coal CCS" = "#dab4c7",
    "d biomass" = "#00931d",
    "d biomass CCS" = "#88c892",
    "e nuclear" = "#ef8e27",
    "f hydro" = "#fdfa28",
    "g wind" = "#3d86f9",
    "h solar" = "#fdd67b",
    "i geothermal" = "#ad440c",
    "j traditional biomass" = "#11d081",
    "energy reduction" = "grey")

# Color scheme for primary fuel costs
PAL_pri_fuelcost <- c( "regional oil" = "#d01c2a",
    "regional natural gas" = "#25a9e0",
    "regional coal" = "#a0237c",
    "regional biomass" = "#00931d",
    "nuclearFuelGenIII" = "#ef8e27")

# Colors for emissions by aggregate sector
PAL_emiss_sector <- c( "liquid systems" = "#d01c2a",
    "gas systems" = "#25a9e0",
    "coal" = "#a0237c",
    "biomass systems" = "#00931d",
    "electricity" = "#ef8e27",
    "hydrogen" = "#fffbac",
    "N fertilizer" = "#11d081",
    "district heat" = "#b14d38",
    "building"="#facda4",
    "industry"="#cef4d1",
    "transportation"="#d0f6f7")

# Aggregated (more that usual) land use
PAL_landuse<- c( "urban" = "#000000",
    "crops" = "#ffd125",
    "pasture (grazed)" = "#aacf22",
    "forest (managed)" = "#41a67a",
    "biomass" = "#00f629",
    "forest (unmanaged)" = "#938e15",
    "shrubs" = "#b14d00",
    "grass/other pasture" = "#ffd081",
    "desert" = "#a7a7a7")

# And hydrogen even
PAL_hydrogen <- c( "gas" = "#25a9e0",
    "gas w/CCS" = "#84e7f9",
    "coal" = "#a0237c",
    "coal w/CCS" = "#dab4c7",
    "biomass" = "#00931d",
    "biomass w/CCS" = "#88c892",
    "nuclear" = "#ef8e27",
    "bulk electricity" = "#fdfa28",
    "wind" = "#3d86f9",
    "solar" = "#fdd67b",
    "energy reduction" = "grey")

# Refined liquids
PAL_refliq <- c( "conventional oil" = "#ab4500",
    "unconventional oil" = "#ff9593",
    "coal" = "#ff2600",
    "coal w/CCS" = "#ff8d78",
    "biomass" = "#00931d",
    "biomass w/CCS" = "#728f72",
    "gas" = "#1633ff",
    "energy reduction" = "grey")

# Color scheme for emissions by aggregate sector
emiss_by_enduse_colors <- c( "buildings" = "#facda4",
    "industry" = "#cef4d1",
    "transportation" = "#d0f6f7",
    "electricity" = "#ea9219",
    "cement" = "#ff230e")

# Color scheme for biomass consumption by use
biouse_colors <- c( "direct buildings" = "#a8d8fe",
"industry" = "#0080d4",
"electricity" = "#ec931a",
"refining" = "#aa4a79",
"hydrogen" = "#fffbac",
"synthetic gas" = "#ff230e", 
"energy reduction" = "grey")

